/* $Id: tearoffmenuitem.gen_h,v 1.3 2001/07/15 13:33:38 murrayc Exp $ */


/* tearoffmenuitem.h
 *
 * Copyright (C) 1998-1999 The Gtk-- Development Team
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <gtk--/menuitem.h>
#include <gtk/gtktearoffmenuitem.h>


namespace Gtk {
//: A menu item used to tear off and reattach its menu.
//- a {\class Gtk::TearoffMenuItem} is a special {Gtk::MenuItem::} which is used
//- to tear off and reattach its menu.
//-
//- When its menu is shown normally, the {\class Gtk::TearoffMenuItem} is drawn
//- as a dotted line indicating that the menu can be torn off. Activating it
//- causes its menu to be torn off and displayed in its own window as a tearoff menu.
//-
//- When its menu is shown as a tearoff menu, the {\class Gtk::TearoffMenuItem} is
//- drawn as a dotted line which has a left pointing arrow graphic indicating that the
//- tearoff menu can be reattached. Activating it will erase the tearoff menu window.
class TearoffMenuItem : public MenuItem {


public:
  typedef TearoffMenuItem          CppObjectType;
  typedef GtkTearoffMenuItem            BaseObjectType;

public:
  //: Returns the underlaying gtk+ object.
  GtkTearoffMenuItem* gtkobj();
  const GtkTearoffMenuItem* gtkobj() const;

  //: Returns true if object is this type.
  static bool isA(Gtk::Object *object);

  virtual ~TearoffMenuItem();

private:

public:

  TearoffMenuItem();



  bool is_torn_off() const { return gtkobj()->torn_off; }



protected:
  // impl functions

};

}
